/**
 * @file   single_thread_monitoring.h
 * @author Giuseppe Congiu
 *         gcongiu@icl.utk.edu
 *
 */

#ifndef __SINGLE_THREAD_MONITORING_H__
#define __SINGLE_THREAD_MONITORING_H__

#include "common.h"

int single_thread(int argc, char *argv[]);

#endif /* End of __SINGLE_THREAD_MONITORING_H__ */
