OUTFILE=TestOutput.csv
echo "Event,1 Kernel Run,.25 Kernel Run,Immediate 2nd Read,Sleep then Read,5 Kernel Run,after .25 vs 1 (exp 1.25),after 5 vs 1 (exp 6.25)" >${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:active_cycles:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:active_cycles_pm:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:active_cycles_sys:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:active_warps:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:active_warps_pm:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:atom_count:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:elapsed_cycles_pm:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:elapsed_cycles_sm:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:elapsed_cycles_sys:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:fb_subp0_read_sectors:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:fb_subp0_write_sectors:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:fb_subp1_read_sectors:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:fb_subp1_write_sectors:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:generic_load:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:generic_store:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:global_atom_cas:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:global_load:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:global_store:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:gred_count:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:inst_executed:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:inst_executed_fma_pipe_s0:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:inst_executed_fma_pipe_s1:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:inst_executed_fma_pipe_s2:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:inst_executed_fma_pipe_s3:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:inst_executed_fp16_pipe_s0:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:inst_executed_fp16_pipe_s1:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:inst_executed_fp16_pipe_s2:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:inst_executed_fp16_pipe_s3:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:inst_executed_fp64_pipe_s0:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:inst_executed_fp64_pipe_s1:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:inst_executed_fp64_pipe_s2:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:inst_executed_fp64_pipe_s3:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:inst_issued0:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:inst_issued1:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:l2_subp0_read_sector_misses:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:l2_subp0_read_sysmem_sector_queries:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:l2_subp0_read_tex_hit_sectors:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:l2_subp0_read_tex_sector_queries:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:l2_subp0_total_read_sector_queries:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:l2_subp0_total_write_sector_queries:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:l2_subp0_write_sector_misses:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:l2_subp0_write_sysmem_sector_queries:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:l2_subp0_write_tex_hit_sectors:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:l2_subp0_write_tex_sector_queries:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:l2_subp1_read_sector_misses:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:l2_subp1_read_sysmem_sector_queries:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:l2_subp1_read_tex_hit_sectors:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:l2_subp1_read_tex_sector_queries:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:l2_subp1_total_read_sector_queries:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:l2_subp1_total_write_sector_queries:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:l2_subp1_write_sector_misses:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:l2_subp1_write_sysmem_sector_queries:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:l2_subp1_write_tex_hit_sectors:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:l2_subp1_write_tex_sector_queries:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:local_load:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:local_store:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:not_predicated_off_thread_inst_executed:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:pcie_rx_active_pulse:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:pcie_tx_active_pulse:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:prof_trigger_00:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:prof_trigger_01:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:prof_trigger_02:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:prof_trigger_03:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:prof_trigger_04:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:prof_trigger_05:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:prof_trigger_06:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:prof_trigger_07:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:shared_atom:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:shared_atom_cas:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:shared_ld_bank_conflict:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:shared_ld_transactions:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:shared_load:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:shared_st_bank_conflict:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:shared_st_transactions:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:shared_store:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:sm_cta_launched:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:tensor_pipe_active_cycles_s0:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:tensor_pipe_active_cycles_s1:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:tensor_pipe_active_cycles_s2:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:tensor_pipe_active_cycles_s3:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:thread_inst_executed:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::event:warps_launched:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:achieved_occupancy:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:atomic_transactions:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:atomic_transactions_per_request:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:cf_executed:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:cf_fu_utilization:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:cf_issued:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:double_precision_fu_utilization:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:dram_read_bytes:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:dram_read_throughput:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:dram_read_transactions:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:dram_utilization:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:dram_write_bytes:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:dram_write_throughput:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:dram_write_transactions:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:eligible_warps_per_cycle:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:flop_count_dp:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:flop_count_dp_add:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:flop_count_dp_fma:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:flop_count_dp_mul:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:flop_count_hp:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:flop_count_hp_add:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:flop_count_hp_fma:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:flop_count_hp_mul:device=0 >>${OUTFILE}
srun -N1 -wa04 ./BlackScholes cuda:::metric:flop_count_sp:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:flop_count_sp_add:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:flop_count_sp_fma:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:flop_count_sp_mul:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:flop_count_sp_special:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:flop_dp_efficiency:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:flop_hp_efficiency:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:flop_sp_efficiency:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:gld_efficiency:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:gld_requested_throughput:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:gld_throughput:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:gld_transactions:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:gld_transactions_per_request:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:global_atomic_requests:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:global_hit_rate:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:global_load_requests:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:global_reduction_requests:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:global_store_requests:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:gst_efficiency:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:gst_requested_throughput:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:gst_throughput:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:gst_transactions:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:gst_transactions_per_request:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:half_precision_fu_utilization:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_bit_convert:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_compute_ld_st:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_control:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_executed:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_executed_global_atomics:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_executed_global_loads:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_executed_global_reductions:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_executed_global_stores:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_executed_local_loads:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_executed_local_stores:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_executed_shared_atomics:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_executed_shared_loads:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_executed_shared_stores:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_executed_surface_atomics:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_executed_surface_loads:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_executed_surface_reductions:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_executed_surface_stores:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_executed_tex_ops:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_fp_16:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_fp_32:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_fp_64:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_integer:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_inter_thread_communication:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_issued:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_misc:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_per_warp:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:inst_replay_overhead:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:ipc:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:issue_slot_utilization:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:issue_slots:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:issued_ipc:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:l2_atomic_throughput:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:l2_atomic_transactions:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:l2_global_atomic_store_bytes:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:l2_global_load_bytes:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:l2_local_global_store_bytes:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:l2_local_load_bytes:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:l2_read_throughput:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:l2_read_transactions:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:l2_surface_load_bytes:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:l2_surface_store_bytes:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:l2_tex_hit_rate:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:l2_tex_read_hit_rate:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:l2_tex_read_throughput:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:l2_tex_read_transactions:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:l2_tex_write_hit_rate:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:l2_tex_write_throughput:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:l2_tex_write_transactions:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:l2_utilization:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:l2_write_throughput:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:l2_write_transactions:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:ldst_executed:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:ldst_fu_utilization:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:ldst_issued:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:local_hit_rate:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:local_load_requests:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:local_load_throughput:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:local_load_transactions:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:local_load_transactions_per_request:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:local_memory_overhead:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:local_store_requests:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:local_store_throughput:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:local_store_transactions:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:local_store_transactions_per_request:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:pcie_total_data_received:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:pcie_total_data_transmitted:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:shared_efficiency:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:shared_load_throughput:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:shared_load_transactions:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:shared_load_transactions_per_request:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:shared_store_throughput:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:shared_store_transactions:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:shared_store_transactions_per_request:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:shared_utilization:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:single_precision_fu_utilization:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:sm_efficiency:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:special_fu_utilization:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:stall_constant_memory_dependency:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:stall_exec_dependency:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:stall_inst_fetch:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:stall_memory_dependency:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:stall_memory_throttle:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:stall_not_selected:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:stall_other:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:stall_pipe_busy:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:stall_sleeping:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:stall_sync:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:stall_texture:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:surface_atomic_requests:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:surface_load_requests:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:surface_reduction_requests:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:surface_store_requests:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:sysmem_read_bytes:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:sysmem_read_throughput:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:sysmem_read_transactions:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:sysmem_read_utilization:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:sysmem_utilization:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:sysmem_write_bytes:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:sysmem_write_throughput:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:sysmem_write_transactions:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:sysmem_write_utilization:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:tensor_precision_fu_utilization:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:tex_cache_hit_rate:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:tex_cache_throughput:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:tex_cache_transactions:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:tex_fu_utilization:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:tex_utilization:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:texture_load_requests:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:warp_execution_efficiency:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::metric:warp_nonpred_execution_efficiency:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000001:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000002:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000003:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000004:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000005:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000006:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000007:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000008:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000009:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600000B:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600000C:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600000D:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600000E:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000011:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000019:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600001A:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600001B:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600001C:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600001D:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600001E:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600001F:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000021:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000023:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000034:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000035:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000068:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000069:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600006B:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600006C:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600006D:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000071:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000072:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000073:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000076:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000077:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600007D:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000083:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000084:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000085:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600008D:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600008E:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600008F:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000090:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000091:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000092:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000093:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000094:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000095:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000096:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000097:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000098:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000099:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600009A:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600009B:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600009C:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600009D:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600009E:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600009F:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000A0:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000A1:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000A2:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000A3:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000A4:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000A5:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000AC:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000AD:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000AE:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000AF:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000B0:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000B1:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000B2:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000B3:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000B4:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000B5:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000B6:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000B7:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000B8:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000B9:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000BA:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000BB:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000BC:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000BD:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000BE:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160000BF:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000115:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000116:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000117:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000126:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000138:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600013D:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600013E:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600015A:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600015C:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000160:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000161:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000162:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000165:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600016C:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600016D:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600016E:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600016F:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000170:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000172:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000173:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000185:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000186:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600018B:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600018C:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600018F:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000190:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000193:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000194:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000195:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600019C:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160001DB:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160001E4:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160001EB:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x160001F0:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600020A:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600020B:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000220:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000221:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000228:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000229:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600022A:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600022B:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600022C:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000234:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000235:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000238:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000239:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600023A:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600023B:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600023C:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x1600023D:device=0 >>${OUTFILE}
#srun -N1 -wa04 ./BlackScholes cuda:::unenum_event:0x16000249:device=0 >>${OUTFILE}
