/**
 * @file   multi_kernel_monitoring.h
 * @author Giuseppe Congiu
 *         gcongiu@icl.utk.edu
 *
 */

#ifndef __MULTI_KERNEL_MONITORING_H__
#define __MULTI_KERNEL_MONITORING_H__

#include "common.h"

int multi_kernel(int argc, char *argv[]);

#endif /* End of __MULTI_KERNEL_MONITORING_H__ */
